proc MakeCompilPrg {numproj prg} {
	global Obj_WinProj Obj_Proj Obj_Dir Process Glob_Pref

	set proj $Obj_Proj($numproj)
	set wprg $Obj_WinProj($numproj,$prg,wbuilder)
	set wlog $Obj_WinProj($numproj,$prg,wlog)
	set wbut $Obj_WinProj($numproj,$prg,wbutaction)

	# Setup the list of targets to compile
	set list_targets {}
	foreach target $Obj_Proj($numproj,$prg,target) {
		if {$Obj_Proj($numproj,$prg,$target)} {
			lappend list_targets $target
		}
	}

	set projpath	$Obj_Dir(Proj)/$Obj_Proj($numproj)
	set pathfiles	$projpath/Src

	# Clean Src Directory  from old Object Modules
	catch { eval exec rm [glob $projpath/Src/*.o] }

	# Clean Builder Directory of Object Modules from selected targets
	foreach target $list_targets {
		regsub "\.$Glob_Pref(FortranExt)$" $target {.o} fileobj
		catch { eval exec rm $projpath/Builder/$fileobj }
	}

	# Get the Absolute name of targets
	set list_tmp {}
	foreach file $list_targets { lappend list_tmp $pathfiles/$file }
	set list_targets $list_tmp

	set trace_id $numproj:$prg
	set trace_type proj

	# Trace for the Compilation Procedure done
	trace variable Process($trace_type,compiling,$trace_id) w TraceProjCompil

	# Launch the Targets Compilation Procedure
	RunCompile $trace_type $trace_id $wprg $list_targets $wlog $wbut

}
proc MakeLinkProgram {numproj prg} {
	global Obj_WinProj Obj_Proj Obj_Dir Glob_Pref env

	set wlog $Obj_WinProj($numproj,$prg,wlog)
	set wbut $Obj_WinProj($numproj,$prg,wbutaction)
	set Builder		$Obj_Dir(Proj)/$Obj_Proj($numproj)/Builder
	set pathfilesobj	$Builder
	set pathprg		$Obj_Dir(Proj)/$Obj_Proj($numproj)/Pgm
	set pathlibusr		$Obj_Dir(Lib)/Lib.ar
	set pathlibcom		$Glob_Pref(SharedLib)
	set home 		$env(HOME)

	set LdOpt [GetLoaderOptionsOnFromFile $Builder/$prg.ldopt {}]
	# If in options, change $HOME item to $home
	if [ string match *\$HOME* $LdOpt ] {
		regsub -all {\$HOME} $LdOpt $home LdOpt
	}
	# If in options, change ~ item to $home
	if [ string match *~* $LdOpt ] {
		regsub -all {~} $LdOpt $home LdOpt
	}


	$wlog configure -state normal
	$wlog insert end \n
	$wlog insert end "Now Linking the Program ..." LINKING
	$wlog insert end \n\n
	$wlog insert end "Loader Options : "
	if { $LdOpt == "" } {
		$wlog insert end "<none>" OK
	} else {
		$wlog insert end $LdOpt OK
	}
	$wlog insert end \n\n
	$wlog see end
	update idletasks

	# Check for Needed Objects Modules :
	set list_obj {}
	foreach target $Obj_Proj($numproj,$prg,target) {
		regsub "\.$Glob_Pref(FortranExt)$" $target {.o} fileobj
		if [ file exists $pathfilesobj/$fileobj ] {
			lappend list_obj $pathfilesobj/$fileobj
		}
	}
	if { [llength $list_obj] != [llength $Obj_Proj($numproj,$prg,target)] } {
		tk_dialog $wlog.d {TkfPW Project Maintainer} \
			"I'm Sorry to inform You\n\
			that Linking Program\n\
			$prg\n\
			didn't succeded\nReason : Missing Target Object Modules" error 0 {See Infos...}
		return
	}

	# Needed Libraries :
	set list_lib {}
	# User's Lib
	foreach lib $Obj_Proj($numproj,$prg,libusr) {
		lappend list_lib  $pathlibusr/$lib
	}
	# Shared Lib
	foreach lib $Obj_Proj($numproj,$prg,libcom) {
		lappend list_lib  $pathlibcom/$lib
	}

	# Linking the Program
	set file_err 	$pathfilesobj/$prg.errlnk
	set file_warn 	$pathfilesobj/$prg.warnlnk
	catch {exec rm $file_err}
	catch {exec rm $file_warn}
	catch {exec rm $pathprg/$prg}
	if [catch {eval exec $Glob_Pref(CompilerName) $LdOpt $list_obj $list_lib -o $pathprg/$prg} err] {
		if [file executable $pathprg/$prg] {
		# Program exists but Warning !
			$wlog insert end \n
			$wlog insert end "Linking Done ! :" OK 
			$wlog insert end \n
			$wlog insert end "Linking Warning ! :" WARNING
			$wlog insert end \n
			$wlog insert end $err
			$wlog see end
			tk_dialog $wlog.d {TkfPW Project Maintainer} \
				"I Inform You that\n\
				the Program $prg\n\
				is Up To Date !\n\
				with Warnings !." info 0 {Check for Warnings ...}
			# Save the Linking Warnings
			WriteVarInFile $err $file_warn
		} else {
		# Program doesn't exists so ERROR !
			$wlog insert end \n
			$wlog insert end "Linking Error ! :" ERROR
			$wlog insert end \n
			$wlog insert end $err
			$wlog see end
			tk_dialog $wlog.d {TkfPW Project Maintainer} \
				"I'm Sorry to inform You\n\
				that Linking Program\n\
				$prg\n\
				didn't succeded\nReason : Linking Error" error 0 {See Infos...}
			# Save the Linking Errors
			WriteVarInFile $err $file_err
#puts "getinfo 8a"
		GetInfoProj $numproj
		GetInfoPrg $numproj $Obj_Proj($numproj) $prg
		}
	} else {
		# Update Infos.
#puts "getinfo 8b"
		GetInfoProj $numproj
		GetInfoPrg $numproj $Obj_Proj($numproj) $prg
		# Program is Up to Date ?
		if { $Obj_Proj($numproj,$prg,status) == {} } {
			$wlog insert end "Linking Done ! :" OK 
			$wlog see end
			tk_dialog $wlog.d {TkfPW Project Maintainer} \
				"I'm Happy to inform You that\n\
				the Program $prg\n\
				is Up To Date !" info 0 {Great !}
		} else {
			# Linking Succeded but ... (may be warnings...)
			if [file executable $pathprg/$prg] {
				$wlog insert end "Linking Done ! :" OK 
				$wlog insert end \n
				$wlog insert end "Status not 100% clean !" ERROR 
				$wlog see end
				tk_dialog $wlog.d {TkfPW Project Maintainer} \
					"I Inform You that\n\
					the Program $prg\n\
					is Up To Date !\n\
					But not 100% clean ." info 0 {Great ?}
		
			}

		}
	}
	$wlog configure -state disabled
}
